<?php

declare(strict_types=1);

namespace Rowlinson\Api\Collections;

use Illuminate\Support\Collection;

/**
 * @template T
 */
final class PaginatedCollection extends Collection
{
    private int $page;
    private int $perPage;
    private int $total;

    /**
     * @param array<T> $items
     * @param int $page The page this collection is of the total
     * @param int $perPage How many items per page was requested
     * @param int $total The total number of items if this collection weren't paginated
     */
    final public function __construct(array $items = [], int $page = 1, int $perPage = 10, int $total = 1)
    {
        parent::__construct($items);

        $this->page = $page;
        $this->perPage = $perPage;
        $this->total = $total;
    }

    public function getPage(): int
    {
        return $this->page;
    }

    public function getPerPage(): int
    {
        return $this->perPage;
    }

    public function getTotal(): int
    {
        return $this->total;
    }

    public function getPageCount(): int
    {
        return intval(ceil($this->total / $this->perPage));
    }

    /**
     * @return PaginatedCollection<T>
     */
    public function map(callable $callback): self
    {
        $keys = array_keys($this->items);
        $items = array_map($callback, $this->items, $keys);

        return new static(array_combine($keys, $items), $this->page, $this->perPage, $this->total);
    }
}
