<?php

declare(strict_types=1);

namespace Rowlinson\Api;

use Rowlinson\Api\Collections\PaginatedCollection;
use Rowlinson\Api\Requests\DeliveryLocation\CreateUpdateRequest;
use Rowlinson\Api\Responses\DeliveryLocation\DeliveryLocationResponse;

class DeliveryLocationClient extends AbstractClient
{
    /**
     * Request the creation of a new delivery address
     */
    public function create(
        string $name,
        string $addressLine1,
        ?string $addressLine2,
        string $town,
        ?string $county,
        string $country,
        string $postcode
    ): void {
        $this->post('delivery_locations', new CreateUpdateRequest(
            $name,
            $addressLine1,
            $addressLine2,
            $town,
            $county,
            $country,
            $postcode
        ));
    }

    /**
     * Request the creation of a new delivery address
     */
    public function edit(
        string $id,
        string $name,
        string $addressLine1,
        ?string $addressLine2,
        string $town,
        ?string $county,
        string $country,
        string $postcode
    ): void {
        $this->put('delivery_locations/' . $id, new CreateUpdateRequest(
            $name,
            $addressLine1,
            $addressLine2,
            $town,
            $county,
            $country,
            $postcode
        ));
    }

    /**
     * Get the delivery locations for the current user
     *
     * @return PaginatedCollection<DeliveryLocationResponse>
     */
    public function getDeliveryLocations(int $page = 1, int $perPage = 10): PaginatedCollection
    {
        $response = $this->get('delivery_locations?' . http_build_query([
            'page' => $page,
            'per_page' => $perPage,
        ]));

        $deliveryResponses = $this->serializer->deserialize(
            (string)$response->getBody(),
            'array<' . DeliveryLocationResponse::class . '>', // @phpstan-ignore-line
            'json'
        );

        return new PaginatedCollection($deliveryResponses, $page, $perPage, $response->getPaginationTotalItems());
    }
}
